"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const ajv_1 = require("ajv");
const ajv = new ajv_1.default({});
/**
 * Tests all the exposed functions of a validator, isValid, validate, and checkValid have consistent return values
 * @param validator the validator to check
 * @param value the value to validate
 * @param expectedError if invalid, these are the errors it should throw
 * @param path the path of the value being passed in
 * @param looserJsonSchemaValidation if true, and validation fails of the validator, it is not also expected to fail for the json schema validator
 */
function checkValidates(validator, value, expectedError, path, looserJsonSchemaValidation = false) {
    (0, chai_1.expect)(validator.validate(value, path)).to.deep.eq(expectedError || [], "#validate");
    if (path !== undefined) {
        return;
    }
    const isExpectedValid = typeof expectedError === "undefined" || expectedError.length === 0;
    (0, chai_1.expect)(validator.isValid(value)).to.eq(isExpectedValid);
    const schema = validator.toJsonSchema();
    const schemaValidator = ajv.compile(schema);
    ajv.validateSchema(schema, true);
    schemaValidator(value);
    if (looserJsonSchemaValidation) {
        if (isExpectedValid)
            (0, chai_1.expect)(!schemaValidator.errors?.length).to.eq(true);
    }
    else {
        (0, chai_1.expect)(!schemaValidator.errors?.length).to.eq(isExpectedValid);
    }
    if (typeof expectedError === "undefined" || expectedError.length === 0) {
        (0, chai_1.expect)(() => validator.checkValid(value)).to.not.throw();
        (0, chai_1.expect)(() => validator.checkValid(value, [0, "abc", 1])).to.not.throw();
    }
    else {
        const errorMatcher = (0, chai_1.expect)(() => validator.checkValid(value, path)).to.throw();
        const messageMatcher = errorMatcher.with.property("message");
        //  message must contain information about every message and path
        expectedError.forEach((err) => {
            messageMatcher.contains(err.message);
            messageMatcher.contains(err.path.join("."));
        });
        // must have the property that indicates it is a failed validation error
        errorMatcher.with.property("isFailedValidationError").eq(true);
        // must have the errors property that is equal to the validation errors
        errorMatcher.with.property("errors").deep.eq(expectedError);
        checkValidates(validator, value, expectedError.map((error) => ({
            ...error,
            path: [0, "abc", 1, ...error.path],
        })), [0, "abc", 1]);
    }
}
exports.default = checkValidates;
//# sourceMappingURL=check-validates.js.map