import { JSONSchema7 } from "json-schema";
import { Infer, ValidationError, ValidationErrorPath, Validator } from "../interfaces";
export interface Keys {
    [key: string]: Validator<unknown>;
}
export declare type ExtractObjectType<TKeys extends Keys> = {
    [K in keyof TKeys]: Infer<TKeys[K]>;
};
interface ObjectValidatorOptions<TKeys extends Keys, TRequiredKeys extends keyof TKeys, AllowUnknownKeys extends boolean | {
    key: Validator<string>;
    value: Validator<any>;
}> {
    readonly keys: TKeys;
    readonly requiredKeys: TRequiredKeys[];
    readonly allowUnknownKeys: AllowUnknownKeys;
}
declare type AllowUnknownKeyObject<TObject extends {}, AllowUnknownKeys extends boolean | {
    key: Validator<string>;
    value: Validator<any>;
}> = AllowUnknownKeys extends {
    key: Validator<string>;
    value: Validator<any>;
} ? {
    [key: string]: Infer<AllowUnknownKeys["value"]>;
} & TObject : AllowUnknownKeys extends false ? TObject : TObject & {
    [key: string]: unknown;
};
declare type WithRequiredKeys<TObject extends {}, TRequiredKeys extends keyof TObject> = {
    [K in keyof TObject]?: TObject[K];
} & {
    [K in keyof Pick<TObject, TRequiredKeys>]: TObject[K];
};
/**
 * Validator that checks a value is an object where each key has a value that matches a given validator.
 */
export default class ObjectValidator<TKeys extends Keys, TRequiredKeys extends keyof TKeys, TAllowUnknown extends boolean | {
    key: Validator<string>;
    value: Validator<any>;
}> extends Validator<AllowUnknownKeyObject<WithRequiredKeys<ExtractObjectType<TKeys>, TRequiredKeys>, TAllowUnknown>> {
    private readonly options;
    constructor(options: ObjectValidatorOptions<TKeys, TRequiredKeys, TAllowUnknown>);
    /**
     * Specify which keys are required for the object to be valid. Replaces any of the existing required keys. By default
     * no keys are required.
     *
     * @param requiredKeys keys that must be present for the object to be valid
     */
    requiredKeys<T extends (keyof TKeys)[]>(...requiredKeys: T | [T]): ObjectValidator<TKeys, T[number], TAllowUnknown>;
    /**
     * Change the behavior of the object validator to allow or disallow keys to be present in the object that are not
     * specified. By default unknown keys are allowed.
     *
     * @param allowUnknownKeys whether to allow keys that are not specified in the object validator to be present
     * @param unknownPropertyValidator validator that is applied to the value of any unknown key that is set
     */
    allowUnknownKeys<TAllow extends boolean | {
        key: Validator<string>;
        value: Validator<any>;
    }>(allowUnknownKeys: TAllow): ObjectValidator<TKeys, TRequiredKeys, TAllow>;
    /**
     * Produce a new object validator by removing the specified keys from the current object validator, similar to TypeScript
     * Omit
     * @param omitted the keys to omit
     */
    omit<TOmitted extends (keyof TKeys)[]>(...omitted: TOmitted): ObjectValidator<Omit<TKeys, TOmitted[number]>, Exclude<TRequiredKeys, TOmitted[number]>, TAllowUnknown>;
    /**
     * Produce a new object validator by selecting only the specified keys from the current set of keys, same as TypeScript Pick
     * @param selected the keys to select
     */
    pick<TSelected extends (keyof TKeys)[]>(...selected: TSelected): ObjectValidator<Pick<TKeys, TSelected[number]>, TSelected[number], TAllowUnknown>;
    /**
     * Add additional properties to the object validator
     * @param additionalKeys the additional keys to add
     */
    extend<TMoreKeys extends Keys>(additionalKeys: TMoreKeys): ObjectValidator<TKeys & TMoreKeys, TRequiredKeys, TAllowUnknown>;
    validate(value: unknown, path?: ValidationErrorPath): ValidationError[];
    isValid(value: unknown): value is AllowUnknownKeyObject<WithRequiredKeys<ExtractObjectType<TKeys>, TRequiredKeys>, TAllowUnknown>;
    _toJsonSchema(): JSONSchema7;
}
export {};
