"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const interfaces_1 = require("../interfaces");
const spread_args_to_array_1 = require("../util/spread-args-to-array");
/**
 * Validator that checks a value is an object where each key has a value that matches a given validator.
 */
class ObjectValidator extends interfaces_1.Validator {
    constructor(options) {
        super();
        this.options = options;
    }
    /**
     * Specify which keys are required for the object to be valid. Replaces any of the existing required keys. By default
     * no keys are required.
     *
     * @param requiredKeys keys that must be present for the object to be valid
     */
    requiredKeys(...requiredKeys) {
        return new ObjectValidator({
            ...this.options,
            requiredKeys: (0, spread_args_to_array_1.spreadArgsToArray)(requiredKeys),
        });
    }
    /**
     * Change the behavior of the object validator to allow or disallow keys to be present in the object that are not
     * specified. By default unknown keys are allowed.
     *
     * @param allowUnknownKeys whether to allow keys that are not specified in the object validator to be present
     * @param unknownPropertyValidator validator that is applied to the value of any unknown key that is set
     */
    allowUnknownKeys(allowUnknownKeys) {
        return new ObjectValidator({
            ...this.options,
            allowUnknownKeys,
        });
    }
    /**
     * Produce a new object validator by removing the specified keys from the current object validator, similar to TypeScript
     * Omit
     * @param omitted the keys to omit
     */
    omit(...omitted) {
        const omittedMap = omitted.reduce((memo, value) => {
            memo[value] = true;
            return memo;
        }, {});
        return new ObjectValidator({
            ...this.options,
            keys: {
                ...Object.fromEntries(Object.entries(this.options.keys).filter(([key]) => !omittedMap[key])),
            },
            requiredKeys: this.options.requiredKeys.filter((requiredKey) => !omittedMap[requiredKey]),
        });
    }
    /**
     * Produce a new object validator by selecting only the specified keys from the current set of keys, same as TypeScript Pick
     * @param selected the keys to select
     */
    pick(...selected) {
        const selectedMap = selected.reduce((memo, value) => {
            memo[value] = true;
            return memo;
        }, {});
        return new ObjectValidator({
            ...this.options,
            keys: {
                ...Object.fromEntries(Object.entries(this.options.keys).filter(([key]) => selectedMap[key])),
            },
            requiredKeys: this.options.requiredKeys.filter((requiredKey) => selectedMap[requiredKey]),
        });
    }
    /**
     * Add additional properties to the object validator
     * @param additionalKeys the additional keys to add
     */
    extend(additionalKeys) {
        return new ObjectValidator({
            ...this.options,
            keys: {
                ...this.options.keys,
                ...additionalKeys,
            },
        });
    }
    validate(value, path = []) {
        const { requiredKeys, keys, allowUnknownKeys } = this.options;
        if (typeof value !== "object" || Array.isArray(value) || value === null) {
            return [{ message: `must be an object`, path, value }];
        }
        let errors = [];
        for (const [key, keyValue] of Object.entries(value)) {
            if (value.hasOwnProperty(key)) {
                if (keys && keys[key] !== undefined) {
                    errors = errors.concat(keys[key].validate(keyValue, [...path, key]));
                }
                else {
                    if (!allowUnknownKeys) {
                        errors.push({
                            message: `encountered unknown key "${key}"`,
                            path,
                            value,
                        });
                    }
                    else {
                        if (typeof allowUnknownKeys !== "boolean") {
                            errors.push(...allowUnknownKeys.key.validate(key, path).map((error) => ({
                                ...error,
                                message: `key "${key}" failed validation: ${error.message}`,
                            })));
                            errors.push(...allowUnknownKeys.value
                                .validate(keyValue, [...path, key])
                                .map((error) => ({
                                ...error,
                                message: `value for key "${key}" failed validation: ${error.message}`,
                            })));
                        }
                    }
                }
            }
        }
        if (requiredKeys) {
            for (const requiredKey of requiredKeys) {
                if (!value.hasOwnProperty(requiredKey)) {
                    errors.push({
                        message: `required key "${String(requiredKey)}" was not defined`,
                        path,
                        value,
                    });
                }
            }
        }
        return errors;
    }
    isValid(value) {
        const { allowUnknownKeys, keys, requiredKeys } = this.options;
        if (typeof value !== "object" || Array.isArray(value) || value === null) {
            return false;
        }
        for (const [key, keyValue] of Object.entries(value)) {
            if (value.hasOwnProperty(key)) {
                if (keys && keys[key] !== undefined) {
                    if (!keys[key].isValid(value[key])) {
                        return false;
                    }
                }
                else {
                    if (!allowUnknownKeys) {
                        return false;
                    }
                    else if (typeof allowUnknownKeys !== "boolean" &&
                        (!allowUnknownKeys.key.isValid(key) ||
                            !allowUnknownKeys.value.isValid(keyValue))) {
                        return false;
                    }
                }
            }
        }
        if (requiredKeys) {
            for (const requiredKey of requiredKeys) {
                if (!value.hasOwnProperty(requiredKey)) {
                    return false;
                }
            }
        }
        return true;
    }
    _toJsonSchema() {
        return {
            type: "object",
            properties: Object.fromEntries(Object.entries(this.options.keys).map(([key, val]) => [
                key,
                val.toJsonSchema(),
            ])),
            additionalProperties: !this.options.allowUnknownKeys
                ? false
                : this.options.allowUnknownKeys === true
                    ? true
                    : this.options.allowUnknownKeys.value.toJsonSchema(),
            propertyNames: typeof this.options.allowUnknownKeys === "boolean"
                ? undefined
                : this.options.allowUnknownKeys.key.toJsonSchema(),
            required: this.options.requiredKeys,
        };
    }
}
exports.default = ObjectValidator;
//# sourceMappingURL=object-validator.js.map